﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System.Drawing.Drawing2D;

namespace SoundSensor_Example {
	public partial class Form1 : Form {
		CommandLineOpen open;
		formCleaner cleaner;
		SoundSensor SND = new SoundSensor();
		private ErrorEventBox errorBox;
		private DataGraphBox splGraphBox;
		Graphics g2;
		Bitmap bm;

		public Form1() {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			InitializeComponent();
		}

		public Form1(String[] commandLine) {
			open = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
			open.commandLine = commandLine;
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e) {

			errorBox = new ErrorEventBox(this);
			splGraphBox = new DataGraphBox(this, "Sound Pressure Level");
			SND.Attach += new Phidget22.Events.AttachEventHandler(SND_attach);
			SND.Detach += new Phidget22.Events.DetachEventHandler(SND_detach);
			SND.Error += new Phidget22.Events.ErrorEventHandler(SND_error);
			SND.SPLChange += new Phidget22.Events.SoundSensorSPLChangeEventHandler(SND_change);
			SND.PropertyChange += SND_propertychange;

			commandLineData phidgetParameters = open.parseCmdLine(); //get command line parameters

			try { //set all the values grabbed from command line.  these values have defaults that are set in ExampleUtils.cs, you can check there to see them.
				SND.Channel = phidgetParameters.Channel; //selects the channel on the device to open
				SND.DeviceSerialNumber = phidgetParameters.SerialNumber; //selects the device or hub to open
				SND.HubPort = phidgetParameters.HubPort; //selects the port on the hub to open
				SND.IsHubPortDevice = phidgetParameters.isHubPortDevice; //is the device a port on a VINT hub?

				if (phidgetParameters.isRemote) //are we trying to open a remote device?
				{
					SND.IsRemote = true;
					Net.EnableServerDiscovery(ServerType.Device); //turn on network scan
					if (phidgetParameters.Password != null && phidgetParameters.ServerName != null)
						Net.SetServerPassword(phidgetParameters.ServerName, phidgetParameters.Password); //set the password if there is one
				} else
					SND.IsLocal = true;

				SND.Open(); //open the device specified by the above parameters
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }

			bm = new Bitmap(graphBox.Width, graphBox.Height);
			g2 = Graphics.FromImage(bm);
			g2.Clear(Color.White);
		}

		private void graphBox_Resize(object sender, EventArgs e) {

			bm = new Bitmap(graphBox.Width, graphBox.Height);
			g2 = Graphics.FromImage(bm);
			g2.Clear(Color.White);
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {
			SND.Attach -= SND_attach;
			SND.Detach -= SND_detach;
			SND.SPLChange -= SND_change;

			Application.DoEvents();
			SND.Close();
		}
		System.Drawing.SolidBrush myBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Red);
		void SND_change(object sender, Phidget22.Events.SoundSensorSPLChangeEventArgs e) {
			try {
				dBTxt.Text = e.DB.ToString("F3");
				dBATxt.Text = e.DBA.ToString("F3");
				dBCTxt.Text = e.DBC.ToString("F3");

				g2.Clear(Color.White);

				float width = bm.Width / e.Octaves.Length;

				SoundSensor ss = sender as SoundSensor;
				int noise = 30;
				int multiplier = 1;
				if (ss.SPLRange == SPLRange.dB_82) {
					noise = 20;
					multiplier = 2;
				}

				for (int i = 0; i < e.Octaves.Length; i++) {
					float height = (float)((e.Octaves[i] - noise) / (SND.MaxdB - noise * multiplier) * bm.Height);

					g2.DrawRectangle(new Pen(Color.Red, 2), i * width, bm.Height - height, width, height);
				}
				graphBox.Invalidate();

				val16k.Text = e.Octaves[9].ToString("F");
				val8k.Text = e.Octaves[8].ToString("F");
				val4k.Text = e.Octaves[7].ToString("F");
				val2k.Text = e.Octaves[6].ToString("F");
				val1k.Text = e.Octaves[5].ToString("F");
				val500.Text = e.Octaves[4].ToString("F");
				val250.Text = e.Octaves[3].ToString("F");
				val125.Text = e.Octaves[2].ToString("F");
				val63.Text = e.Octaves[1].ToString("F");
				val32.Text = e.Octaves[0].ToString("F");

				splGraphBox.addData(e.DB, SND.DataInterval);

			} catch (PhidgetException ex) { MessageBox.Show(ex.ToString()); }
		}

		private void graphBox_Paint(object sender, PaintEventArgs e) {
			e.Graphics.DrawImage(bm, 0, 0);
		}

		void SND_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox2.FillPhidgetInfo((Phidget)sender);
			SoundSensor attachedDevice = (SoundSensor)sender;

			splGraphBox.SetTitle(DataGraphBox.GenerateTitle("Sound Pressure Level", (Phidget)attachedDevice));

			try {
				attachedDevice.SPLRange = SPLRange.dB_102;
				dataIntervalTrk.Unit = "ms";
				dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
				dataIntervalTrk.minTxt = attachedDevice.MinDataInterval.ToString();
				dataIntervalTrk.maxTxt = attachedDevice.MaxDataInterval.ToString();
				dataIntervalTrk.Minimum = attachedDevice.MinDataInterval;
				dataIntervalTrk.Maximum = attachedDevice.MaxDataInterval;
				dataIntervalTrk.Value = attachedDevice.DataInterval;
				changeTriggerTrk.Unit = "dB";
				changeTriggerTrk.minTxt = attachedDevice.MinSPLChangeTrigger.ToString();
				changeTriggerTrk.maxTxt = attachedDevice.MaxSPLChangeTrigger.ToString();
				changeTriggerTrk.Minimum = attachedDevice.MinSPLChangeTrigger;
				changeTriggerTrk.Maximum = attachedDevice.MaxSPLChangeTrigger;
				changeTriggerTrk.Value = attachedDevice.SPLChangeTrigger;
			} catch (PhidgetException ex) {
				errorBox.addMessage("Error initializing device: " + ex.Message);
			}

			splRangeCombo.DataSource = typeof(SPLRange).ToList();
			splRangeCombo.SelectedIndex = 0;

			settingsBox.Visible = true;
			outputBox.Visible = true;
			testUtilityBox.Visible = true;
			frequencyPanel.Visible = true;
			groupBox1.Visible = true;
			frequencyTrk.Value = 1000;
			bckgndWorker1Status.Text = frequencyTrk.Value.ToString() + " Hz";
		}

		void SND_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox2.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			testUtilityBox.Visible = false;
			frequencyPanel.Visible = false;
			groupBox1.Visible = false;
		}

		private void SND_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation)
				splGraphBox.addGap();
		}

		private void SND_propertychange(object sender, PropertyChangeEventArgs e)
		{

			switch (e.PropertyName)
			{
				case "DataInterval":
					if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
					{
						dataIntervalTrk.Value = SND.DataInterval;
						updateGraphDataInterval();
					}
					break;
				case "SPLChangeTrigger":
					changeTriggerTrk.Value = SND.SPLChangeTrigger;
					break;
				case "DataRate":
					if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
					{
						dataIntervalTrk.Value = SND.DataRate;
						updateGraphDataInterval();
					}
					break;
			}
		}

		private void FrequencyTrk_Scroll(object sender, EventArgs e) {
			bckgndWorker1Status.Text = frequencyTrk.Value.ToString() + " Hz";
		}

		private void AsyncWorker_Click(object sender, EventArgs e) {
			AsyncWorker.Enabled = false;
			if (!backgroundWorker1.IsBusy)
				backgroundWorker1.RunWorkerAsync(frequencyTrk.Value);
		}

		private void backgroundWorker1_DoWork(object sender, DoWorkEventArgs e) {
			Console.Beep((int)e.Argument, 2500);
		}

		private void backgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
			AsyncWorker.Enabled = true;
		}

		private void dataIntervalTrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					SND.DataInterval = (int)dataIntervalTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}

		private void updateGraphDataInterval()
		{
			try
			{
				if (SND.SPLChangeTrigger == 0)
					splGraphBox.DataInterval = (1000.0 / SND.DataRate);
				else
					splGraphBox.DataInterval = null;
			}
			catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}
		private void dataIntervalTrk_MouseUp(object sender, MouseEventArgs e) {
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					SND.DataInterval = (int)Math.Round(dataIntervalTrk.Value);
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data interval: " + ex.Message);
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					SND.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex)
				{
					errorBox.addMessage("Error setting data rate: " + ex.Message);
				}
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}


		private void changeTriggertrk_Scroll(object sender, EventArgs e) {
			if (MouseButtons != MouseButtons.Left) {
				try {
					SND.SPLChangeTrigger = changeTriggerTrk.Value;
				} catch (Exception ex) {
					errorBox.addMessage("Error setting change trigger: " + ex.Message);
				}
			}

			changeTriggerTrk.trkText += " A";
		}

		private void changeTriggertrk_MouseUp(object sender, MouseEventArgs e) {
			try {
				SND.SPLChangeTrigger = changeTriggerTrk.Value;
			} catch (Exception ex) {
				errorBox.addMessage("Error setting change trigger: " + ex.Message);
			}
		}

		private void comboBox1_SelectedIndexChanged(object sender, EventArgs e) {
			ComboBox box = (ComboBox)sender;
			if (box.SelectedIndex == 0) {
				SND.SPLRange = SPLRange.dB_102;
			} else {
				SND.SPLRange = SPLRange.dB_82;
			}
		}

		private void splGraphButton_Click(object sender, EventArgs e) {
			if (!splGraphBox.Visible)
				splGraphBox.Show();
		}

		private void dataIntervalTrk_SetClick(object sender, EventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					SND.DataInterval = (int)dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					SND.DataRate = dataIntervalTrk.Value;
					updateGraphDataInterval();
				}
				catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
			}

		}

		private void dataIntervalTrk_SetEnter(object sender, KeyEventArgs e)
		{
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				if (e.KeyCode == Keys.Enter)
				{
					try
					{
						SND.DataInterval = (int)dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				}
			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				if (e.KeyCode == Keys.Enter)
				{
					try
					{
						SND.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					}
					catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			}
		}

		private void changeTriggerTrk_SetClick(object sender, EventArgs e) {
			try {
				SND.SPLChangeTrigger = changeTriggerTrk.Value;
			} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
		}

		private void changeTriggerTrk_SetEnter(object sender, KeyEventArgs e) {
			if (e.KeyCode == Keys.Enter) {
				try {
					SND.SPLChangeTrigger = changeTriggerTrk.Value;
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}

        private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e)
        {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE)
			{
				try
				{
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.isFixedRange = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.Maximum = SND.MaxDataRate;
					dataIntervalTrk.Minimum = SND.MinDataRate;
					dataIntervalTrk.maxTxt = SND.MaxDataRate.ToString();
					dataIntervalTrk.minTxt = SND.MinDataRate.ToString("F3");
					dataIntervalTrk.trkText = SND.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = SND.DataRate;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data rate: " + ex.Message); }

			}
			else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL)
			{
				try
				{
					dataIntervalTrk.isIntValue = true;
					dataIntervalTrk.isFixedRange = true;
					dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.Maximum = SND.MaxDataInterval;
					dataIntervalTrk.Minimum = SND.MinDataInterval;
					dataIntervalTrk.maxTxt = SND.MaxDataInterval.ToString();
					dataIntervalTrk.minTxt = SND.MinDataInterval.ToString();
					dataIntervalTrk.trkText = SND.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = SND.DataInterval;
				}
				catch (Exception ex)
				{ errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
    }
}